@@ -1,7 +1,8 @@
+use core::fmt;
 use core::marker;
 
 use crate::integer_encoding::{IntegerEncoding, UsizeEncoding};
-use musli::en::{Encoder, PackEncoder, PairEncoder, SequenceEncoder, VariantEncoder};
+use musli::en::{Encoder, PackEncoder, PairEncoder, SequenceEncoder};
 use musli_binary_common::writer::Writer;
 
 /// A vaery simple encoder suitable for storage encoding.
@@ -35,6 +36,7 @@ where
     I: IntegerEncoding,
     L: UsizeEncoding,
 {
+    type Ok = ();
     type Error = W::Error;
 
     type Pack = Self;
@@ -45,9 +47,14 @@ where
     type Tuple = Self;
     type Variant = Self;
 
+    #[inline]
+    fn expecting(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        write!(f, "type supported by the storage encoder")
+    }
+
     #[inline]
     fn encode_unit(self) -> Result<(), Self::Error> {
-        SequenceEncoder::finish(self.encode_sequence(0)?)
+        SequenceEncoder::end(self.encode_sequence(0)?)
     }
 
     #[inline]
@@ -220,6 +227,7 @@ where
     I: IntegerEncoding,
     L: UsizeEncoding,
 {
+    type Ok = ();
     type Error = W::Error;
     type Encoder<'this> = StorageEncoder<&'this mut W, I, L> where Self: 'this;
 
@@ -229,7 +237,7 @@ where
     }
 
     #[inline]
-    fn finish(self) -> Result<(), Self::Error> {
+    fn end(self) -> Result<(), Self::Error> {
         Ok(())
     }
 }
@@ -240,16 +248,17 @@ where
     I: IntegerEncoding,
     L: UsizeEncoding,
 {
+    type Ok = ();
     type Error = W::Error;
-    type Next<'this> = StorageEncoder<&'this mut W, I, L> where Self: 'this;
+    type Encoder<'this> = StorageEncoder<&'this mut W, I, L> where Self: 'this;
 
     #[inline]
-    fn encode_next(&mut self) -> Result<Self::Next<'_>, Self::Error> {
+    fn next(&mut self) -> Result<Self::Encoder<'_>, Self::Error> {
         Ok(StorageEncoder::new(&mut self.writer))
     }
 
     #[inline]
-    fn finish(self) -> Result<(), Self::Error> {
+    fn end(self) -> Result<(), Self::Error> {
         Ok(())
     }
 }
@@ -260,44 +269,23 @@ where
     I: IntegerEncoding,
     L: UsizeEncoding,
 {
+    type Ok = ();
     type Error = W::Error;
     type First<'this> = StorageEncoder<&'this mut W, I, L> where Self: 'this;
     type Second<'this> = StorageEncoder<&'this mut W, I, L> where Self: 'this;
 
     #[inline]
-    fn encode_first(&mut self) -> Result<Self::First<'_>, Self::Error> {
+    fn first(&mut self) -> Result<Self::First<'_>, Self::Error> {
         Ok(StorageEncoder::new(&mut self.writer))
     }
 
     #[inline]
-    fn encode_second(&mut self) -> Result<Self::Second<'_>, Self::Error> {
+    fn second(&mut self) -> Result<Self::Second<'_>, Self::Error> {
         Ok(StorageEncoder::new(&mut self.writer))
     }
 
     #[inline]
-    fn finish(self) -> Result<(), Self::Error> {
+    fn end(self) -> Result<(), Self::Error> {
         Ok(())
     }
 }
-
-impl<W, I, L> VariantEncoder for StorageEncoder<W, I, L>
-where
-    W: Writer,
-    I: IntegerEncoding,
-    L: UsizeEncoding,
-{
-    type Error = W::Error;
-
-    type VariantTag<'this> = StorageEncoder<&'this mut W, I, L> where Self: 'this;
-    type VariantValue = Self;
-
-    #[inline]
-    fn encode_variant_tag(&mut self) -> Result<Self::VariantTag<'_>, Self::Error> {
-        Ok(StorageEncoder::new(&mut self.writer))
-    }
-
-    #[inline]
-    fn encode_variant_value(self) -> Result<Self::VariantValue, Self::Error> {
-        Ok(self)
-    }
-}
